// Add bubble to the top of the page.
var gBubble = null;
var gTopLinks = null; 
var gBottomLinks = null; 
var gTranslationsDiv = null;

var gTranslations = [];
var gTranslationWordDivs = [];
var gSelectedTranslationIndex = 0;
var gOriginalWord = '';
var gNumLookupsAfterQuiz = 0;
var gNumLookupsInQuiz = 5;
var gDisabled = false;

var gStorage = null;

chrome.storage.onChanged.addListener(function(changes, namespace) {
    for (key in changes) {
        var storageChange = changes[key];
        gStorage[key] = changes[key].newValue;
        if (key=='num_def_before_quiz') {
            gNumLookupsInQuiz = parseInt(gStorage['num_def_before_quiz'], 10);
        }
        if (key=='disabled') {
            gDisabled = gStorage['disabled']=='true';
            if (gDisabled) {
                closeBubble();
            }
        }
    }
});

chrome.storage.sync.get({
    native_language: "eng",
    foreign_language: "deu",
    num_def_before_quiz: "5",
    disabled: "false"
}, function(items) {
    storage = {}
    storage['native_language'] = items.native_language;
    storage['foreign_language'] = items.foreign_language;
    storage['num_def_before_quiz'] = items.num_def_before_quiz;
    storage['disabled'] = items.disabled;
    gNumLookupsInQuiz = parseInt(items.num_def_before_quiz, 10);
    gDisabled = items.disabled=='true';
    gStorage = storage
});

function addWordToTranslations(sourceWord, translatedWord) {
    var start = Math.max(0, gTranslations.length - 2 * gNumLookupsInQuiz);
    for (var index=start; index<gTranslations.length; index++) {
        if (gTranslations[index][0] == sourceWord) {
            return;
        }
    }
    gTranslations.push([sourceWord, translatedWord]);
    gNumLookupsAfterQuiz++;
}


function closeBubble() {
    if (!gBubble) {
        return;
    }
    gBubble.innerHTML = '';
    gBubble.style.visibility = 'hidden';
    // if we have a translation, save in the gTranslations array
    if (gTranslationWordDivs.length > gSelectedTranslationIndex) {
        var selectedTranslationWord;
        if (gSelectedTranslationIndex == gTranslationWordDivs.length-1) {
            selectedTranslationWord = gTranslationWordDivs[gSelectedTranslationIndex].firstElementChild.value;
        } else {
            selectedTranslationWord = gTranslationWordDivs[gSelectedTranslationIndex].innerHTML;
        }
        addWordToTranslations(gOriginalWord, selectedTranslationWord);
        gTranslationWordDivs = [];
    }
    gBubble.parentNode.removeChild(gBubble);
    gBubble = null;
}


// Lets listen to mouseup DOM events.
document.addEventListener('mouseup',
    function (e) {
        if (gDisabled) {
            return;
        }
        if (gBubble && gBubble.style.visibility == 'hidden') {
            return;
        }
        if (gStorage == null) {
            return;
        }
        /*
        if (e.target.parentElement == gTopLinks || e.target.parentElement == gBottomLinks) {
            return;
        }
        */
        if (e.target.id == "bubble_other") {
            gTranslationWordDivs[gSelectedTranslationIndex].style.border = '';
            gSelectedTranslationIndex = gTranslationWordDivs.length - 1;
            gTranslationWordDivs[gSelectedTranslationIndex].style.border = '1px solid #888';
            return;
        }
        if (e.target.parentElement == gTranslationsDiv) {
            var elementId = parseInt(e.srcElement.id, 10);
            if (gSelectedTranslationIndex != elementId) {
                gTranslationWordDivs[gSelectedTranslationIndex].style.border = '';
                gSelectedTranslationIndex = elementId;
                gTranslationWordDivs[gSelectedTranslationIndex].style.border = '1px solid #888';
            }
            return;
        }

        // see if we are clicking in the bubble region
        if (gBubble) {
            var node = e.target.parentElement;
            while (node) {
                if (node == gBubble) {
                    return;
                }
                node = node.parentNode;
            }
        }

        // close bubble
        if (gBubble) {
            closeBubble()
            return;
        }

        var selection = window.getSelection().toString();
        if (selection.length > 0) {
            gBubble = document.createElement('div');
            gBubble.setAttribute('class', 'selection_bubble');

            var originalWords = XRegExp.matchChain(selection, [XRegExp("[\\p{Alphabetic}\\p{Nd}\\{Pc}\\p{M}]+", "g")]);
            gOriginalWord = originalWords[0];

            var selectedElement = window.getSelection();
            var selectedRange = selectedElement.getRangeAt(0); //get the text range
            var selectedRect = selectedRange.getBoundingClientRect();

            /*
            gTranslations = [['sind', 'are'], ['von', 'from'], ['Korrektur', 'correction'], ['alte', 'old'], ['eine', 'a']];
            triggerQuiz(e);
            */

            /*
            gOriginalWord = 'alt';
            words = ['old', 'ancient', 'aged'];
            renderBubble(selectedRect, words);
            */

            if (gNumLookupsAfterQuiz >= gNumLookupsInQuiz) {
                triggerLookup(selectedRect, true);
                triggerQuiz(e);
                gNumLookupsAfterQuiz = 0;
            } else {
                triggerLookup(selectedRect, true);
            }
        }
    },
    false);

function getShortLanguage(threeLetterLanguage) {
    if (threeLetterLanguage == 'eng') { return 'en'; }
    if (threeLetterLanguage == 'deu') { return 'de'; }
    if (threeLetterLanguage == 'fra') { return 'fr'; }
    if (threeLetterLanguage == 'spa') { return 'es'; }
    if (threeLetterLanguage == 'rus') { return 'ru'; }
    return 'en';
}

function makeTranslationWordDiv(index, word) {
    var translationWordDiv = document.createElement('div');
    gTranslationWordDivs.push(translationWordDiv);
    translationWordDiv.setAttribute('class', 'translation');
    translationWordDiv.setAttribute('id', index);
    translationWordDiv.innerHTML = word;
    return translationWordDiv;
}

// Move that gBubble to the appropriate location.
function renderBubble(selectedRect, words) {
    document.body.appendChild(gBubble);
    var srcLanguageShort = getShortLanguage(gStorage['foreign_language']);
    var dstLanguageShort = getShortLanguage(gStorage['native_language']);

    gTranslationsDiv = document.createElement('div');
    gTranslationsDiv.setAttribute('class', 'translations');
   
    var translationText = gOriginalWord + ':';
    gTranslationsDiv.innerHTML = translationText;

    gTranslationWordDivs = [];
    for (var index=0; index<Math.min(5, words.length); index++) {
        var translationWordDiv = makeTranslationWordDiv(index, words[index]);
        gTranslationsDiv.appendChild(translationWordDiv);
    }
    var translationWordDiv = makeTranslationWordDiv(index, '<input onClick="this.select();" type="text" id="bubble_other" value="other">');
    gTranslationsDiv.appendChild(translationWordDiv);

    gSelectedTranslationIndex = 0;
    gTranslationWordDivs[0].style.border = '1px solid #888';

    var links = document.createElement('div');
    links.setAttribute('class', 'links');

    gTopLinks = document.createElement('div');
    gTopLinks.setAttribute('class', 'toplinks');
    var linksText = '';
    linksText += '<a target="_blank" href="https://glosbe.com/' + srcLanguageShort + '/' + dstLanguageShort + '/' + gOriginalWord + '">';
    linksText += 'ADD or DETAIL glosbe.com&rarr;</a><br>';

    linksText += '<a target="_blank" href="https://translate.google.com/#' + srcLanguageShort + '/' + dstLanguageShort + '/' + gOriginalWord + '">';
    linksText += 'TRANSLATE at google&rarr;</a><br>';

    linksText += '<a target="_blank" href="https://www.google.com/?gws_rd=ssl#q=' + gOriginalWord + '">GOOGLE "' + gOriginalWord + '"&rarr;</a><br>';
    gTopLinks.innerHTML = linksText;

    gBottomLinks = document.createElement('div');
    gBottomLinks.setAttribute('class', 'bottomlinks');
    var optionsText = '';
    var optionsLink = chrome.extension.getURL("options.html");
    optionsText += '<a target="_blank" href="' + optionsLink + '">';
    optionsText += 'OPTIONS&rarr;</a><br>';
    optionsText += '<a id="disable_link">';
    optionsText += 'DISABLE&rarr;</a>';
    gBottomLinks.innerHTML = optionsText;

    links.appendChild(gTopLinks);
    links.appendChild(gBottomLinks);

    gBubble.appendChild(gTranslationsDiv);
    gBubble.appendChild(links);

    var maxHeight = Math.max(gTranslationsDiv.offsetHeight, links.offsetHeight);
    gTranslationsDiv.style.height = maxHeight + 'px';
    links.style.height = maxHeight + 'px';

    positionBubble(selectedRect);

    var disable_link = document.getElementById('disable_link');
    disable_link.onclick = function() {
        gDisabled = true;
        closeBubble();
        chrome.storage.sync.set({
            disabled: 'true'
        });
    }
}

function positionBubble(selectedRect) {
    var adjust = gBubble.offsetHeight + 4;

    var scrollLeft = document.documentElement.scrollLeft || document.body.scrollLeft;
    var scrollTop = document.documentElement.scrollTop || document.body.scrollTop;
    gBubble.style.left = (selectedRect.left + scrollLeft) + 'px';
    gBubble.style.top = (selectedRect.top + scrollTop - adjust) + 'px';
    gBubble.style.visibility = 'visible';
}


function isUpper(character) {
    return character == character.toUpperCase();
}


function createLookupUrl(word) {
    return url = "https://glosbe.com/gapi/translate" +
                "?from=" + gStorage['foreign_language'] +
                "&dest=" + gStorage['native_language'] +
                "&format=json" +
                "&phrase=" + encodeURI(word);
}


function triggerLookup(selectedRect, tryLowerCase) {
    var url = createLookupUrl(gOriginalWord);
    var xmlHttp = new XMLHttpRequest();
    xmlHttp.timeout = 5000; // milli-seconds
    xmlHttp.ontimeout = function() { handleTimeout(url, selectedRect); }
    xmlHttp.onreadystatechange = function f() { handleWordLookup(selectedRect, xmlHttp, tryLowerCase); }
    xmlHttp.onerror = function f() { handleError(url, selectedRect); }
    xmlHttp.open("GET", url, true);
    xmlHttp.send(null);

}


function triggerQuiz(e) {
    chrome.runtime.sendMessage({
        type:'quiz',
        translations:gTranslations.slice(gTranslations.length-gNumLookupsInQuiz, gTranslations.length)
    });
}

function handleTimeout(url, selectedRect) {
    document.body.appendChild(gBubble);
    gBubble.innerHTML = '<b>Lookup took over 5 seconds!</b><br>Try again or disable <i>Reading Vocabulary Extension</i><br>Timeout on url:<a href="' + url + '">' + url + '</a>';
    gBubble.style.padding = '10px';
    positionBubble(selectedRect);
}

function handleError(url, selectedRect) {
    document.body.appendChild(gBubble);
    gBubble.innerHTML = 'ERROR!<br>Could not connect to:<br>"' + url + '"';
    gBubble.style.padding = '10px';
    positionBubble(selectedRect);
}

function handleWordLookup(selectedRect, xmlHttp, tryLowerCase) {
    if (xmlHttp.readyState == 4 && xmlHttp.status == 200) {
        var lookupResult = JSON.parse(xmlHttp.responseText);
        var words;
        if (lookupResult.result == "ok") {
            words = [];
            if (tryLowerCase && lookupResult.tuc.length == 0 && isUpper(gOriginalWord[0])) {
                // if we failed the first time, lookup the lower case version
                gOriginalWord = gOriginalWord.toLowerCase();
                triggerLookup(selectedRect, false);
                return;
            }
            else {
                // copy results into words array
                for (var translationKey in lookupResult.tuc) {
                    var translation = lookupResult.tuc[translationKey];
                    if ('phrase' in translation) {
                        words.push(translation['phrase'].text)
                    }
                }
            }
        }
        if (words.length == 0) {
            // if we couldn't find a translation, just use the original word
            words.push(gOriginalWord);
        }
        renderBubble(selectedRect, words);
    }
}

